import datetime
from flask import Blueprint, request, jsonify, make_response, json
from app.utils import token_required, paginated_query, delete_record, single_record_query, add_logo, get_logo_from_logotable
from app import db
from . import models
from app.devices.models import Device
from app.rooms.models import Room
from sqlalchemy import desc
from .schemas import equipment_schema

equipment_bp = Blueprint('equipment', __name__)

@equipment_bp.route('/api/v1/add/equipment', methods=['POST'])
@token_required(1, 2, 3)
def create_equipment(current_user):
    try:
        equipment_dict = json.loads(request.form.get('formData'))
        if equipment_dict.get('device_name') == "Device not available":
            equipment_dict['device_name'] = ""
        
        equipment_data = {
            'equipment_name': equipment_dict['equipment_name'].strip(),
            'serial_number': equipment_dict['serial_number'],
            'status': equipment_dict.get('status', 'Active'),
            'site_id': request.args.get('site_id'),
            'device_id': equipment_dict.get('device_id', ''),
            'description': equipment_dict.get('description')
        }
        
        loaded_equipment_data = equipment_schema.load(equipment_data)
        db.session.add(models.Equipment(**loaded_equipment_data))
        db.session.commit()
        return make_response(jsonify({'message': 'Equipment added successfully', 'status': 'Success'}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, 'message': 'Failed to add Equipment', 'status': 'error'}), 500)

@equipment_bp.route('/api/v1/get/equipment', methods=['GET'])
@token_required(1, 2, 3)
def get_equipment_list(current_user):
    try:
        equipment_query = models.Equipment.query.order_by(desc(models.Equipment.equipment_id))
        equipment_paginated, data = paginated_query(equipment_query, [models.Equipment.equipment_name], models.Equipment.site_id)
        
        serialized_equipment = []
        for eq in equipment_paginated:
            device_name = ""
            room_name = ""
            device = Device.query.filter_by(device_id=eq.device_id).first()
            if device:
                device_name = device.device_name
                room = Room.query.filter_by(id=device.room_id).first()
                if room:
                    room_name = room.room_name
                    
            serialized_equipment.append({
                'equipment_id': eq.equipment_id,
                'equipment_name': eq.equipment_name,
                'serial_number': eq.serial_number,
                'status': eq.status,
                'site_id': eq.site_id,
                'device_id': eq.device_id,
                'device_name': device_name,
                'room_name': room_name,
                'description': eq.description,
                'created_at': eq.created_at.strftime('%Y-%m-%d %H:%M:%S') if eq.created_at else None,
                'updated_at': eq.updated_at.strftime('%Y-%m-%d %H:%M:%S') if eq.updated_at else None
            })
            
        return make_response(jsonify({'equipment': serialized_equipment, 'data': data}), 200)
    except Exception as e:
        return make_response(jsonify({'message': f'Error: {str(e)}', 'status': 'error'}), 500)

@equipment_bp.route('/api/v1/get/equipment_by_id/<int:id>', methods=['GET'])
@token_required(1, 2, 3)
def get_equipment(current_user, id):
    try:
        equipment = single_record_query(models.Equipment, models.Equipment.equipment_id, id, current_user)
        if equipment:
            device = Device.query.filter_by(device_id=equipment.device_id).first()
            device_name = device.device_name if device else None
            
            return make_response(jsonify({'equipment': {
                'equipment_id': equipment.equipment_id,
                'equipment_name': equipment.equipment_name,
                'serial_number': equipment.serial_number,
                'status': equipment.status,
                'site_id': equipment.site_id,
                'device_id': equipment.device_id,
                'device_name': device_name,
                'description': equipment.description,
                'created_at': equipment.created_at.strftime('%Y-%m-%d %H:%M:%S') if equipment.created_at else None,
                'updated_at': equipment.updated_at.strftime('%Y-%m-%d %H:%M:%S') if equipment.updated_at else None
            }}), 200)
        return make_response(jsonify({'message': 'Equipment not found', 'status': 'error'}), 404)
    except Exception as e:
        return make_response(jsonify({'message': f'Error: {e}', 'status': 'error'}), 500)

@equipment_bp.route('/api/v1/update_equipment/<int:equipment_id>', methods=['PUT'])
@token_required(1, 2, 3)
def update_equipment(current_user, equipment_id):
    try:
        equipment_dict = json.loads(request.form.get('formData'))
        equipment_to_update = single_record_query(models.Equipment, models.Equipment.equipment_id, equipment_id, current_user)
        
        if not equipment_to_update:
            return make_response(jsonify({'message': 'Equipment not found', 'status': 'error'}), 404)
            
        if equipment_dict.get('device_name') == "Device not available" or equipment_dict.get('device_name') is None:
            equipment_dict['device_name'] = ""
            
        for key, value in equipment_schema.load({
            'equipment_name': equipment_dict['equipment_name'].strip(),
            'serial_number': equipment_dict.get('serial_number', equipment_to_update.serial_number),
            'status': equipment_dict.get('status', equipment_to_update.status),
            'site_id': request.args.get('site_id', equipment_to_update.site_id),
            'device_id': str(equipment_dict.get('device_id', equipment_to_update.device_id)),
            'description': equipment_dict.get('description', equipment_to_update.description)
        }).items():
            setattr(equipment_to_update, key, value)
        
        db.session.commit()
        return make_response(jsonify({'message': 'Equipment updated successfully', 'status': 'Success'}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, 'message': 'Failed to update Equipment', 'status': 'error'}), 500)

@equipment_bp.route('/api/v1/delete-equipment/<id>', methods=['DELETE'])
@token_required(1, 2, 3)
def delete_equipment(current_user, id):
    return delete_record(models.Equipment, current_user, id, "Equipment", "equipment_id") 